!*************************************************************************
! This file is part of 
!
!	HiggsBounds 1.0
!
! by Philip Bechtle, Oliver Brein, Sven Heinemyer, Georg Weiglein
!    and Karina E. Williams.
!
!  Journal Reference: e-Print: arXiv:0811.4169 [hep-ph], submitted to CPC.
!  Web site: http://www.ippp.dur.ac.uk/HiggsBounds
!
!27/01/2009
!*************************************************************************
#include "tables-nhmax.F"
#include "LEP-bound.F"
#include "TEV-bound.F"


*************************************************************************
	subroutine run_HiggsBounds_effC(NHIGGS,Mh,GammaTotal, 
     & g2hjbb,g2hjtautau,g2hjWW,g2hjZZ, 
     & g2hjgaga,g2hjgg,g2hjhiZ, 
     & BR_hjhihi, 
     & HBresult,chan, 
     & obsratio, ncombined ) 
*************************************************************************
	implicit none
! input:
	integer NHIGGS,i,j,k
	double precision Mh(NHIGGS),GammaTotal(NHIGGS)
	double precision g2hjbb(NHIGGS),g2hjtautau(NHIGGS),
     &		g2hjWW(NHIGGS),g2hjZZ(NHIGGS),g2hjgaga(NHIGGS),g2hjgg(NHIGGS),
     &		g2hjhiZ(NHIGGS,NHIGGS),BR_hjhihi(NHIGGS,NHIGGS)
! output:
	integer HBresult,chan,ncombined
	double precision obsratio
! internal:
!  Tevatron partonic CS
	double precision CS_tev_gg_hj_ratio(NHIGGS),CS_tev_bb_hj_ratio(NHIGGS),
     & 		CS_tev_bg_hjb_ratio(NHIGGS),
     & 		CS_tev_ud_hjWp_ratio(NHIGGS),CS_tev_cs_hjWp_ratio(NHIGGS),
     & 		CS_tev_ud_hjWm_ratio(NHIGGS),CS_tev_cs_hjWm_ratio(NHIGGS),
     & 		CS_tev_dd_hjZ_ratio(NHIGGS),CS_tev_uu_hjZ_ratio(NHIGGS),
     & 		CS_tev_ss_hjZ_ratio(NHIGGS),CS_tev_cc_hjZ_ratio(NHIGGS),
     & 		CS_tev_bb_hjZ_ratio(NHIGGS)
!  LEP CS ratios & Tevatron hadronic CS
        double precision CS_lep_hjZ_ratio(NHIGGS),
     &		CS_lep_hjhi_ratio(NHIGGS,NHIGGS)

	double precision g2hjhiZ_compact((NHIGGS*(NHIGGS+1))/2)
	double precision CS_lep_hjhi_ratio_compact((NHIGGS*(NHIGGS+1))/2)

	double precision CS_tev_pp_hj_ratio(NHIGGS),
     &		CS_tev_pp_hjb_ratio(NHIGGS),CS_tev_pp_hjW_ratio(NHIGGS),
     &		CS_tev_pp_hjZ_ratio(NHIGGS),CS_tev_pp_vbf_ratio(NHIGGS)
!  BRs
       	double precision BR_hjbb(NHIGGS),BR_hjtautau(NHIGGS),BR_hjWW(NHIGGS),
     &		BR_hjgaga(NHIGGS)

! translate g2hjhiZ to g2hjhiZ_compact
        k=0
        do j=1,NHIGGS
          do i=1,NHIGGS
            if(i .le. j) then
              k=k+1
              g2hjhiZ_compact(k)=g2hjhiZ(j,i)
            endif
          enddo
        enddo


	  call calc_CS_and_BR_from_couplings(NHIGGS,Mh,GammaTotal,
     &		g2hjbb, g2hjtautau, g2hjWW, g2hjZZ,
     &          g2hjgaga, g2hjgg, g2hjhiZ_compact,
     &		CS_lep_hjZ_ratio,CS_lep_hjhi_ratio_compact,
     &		CS_tev_gg_hj_ratio,CS_tev_bb_hj_ratio,CS_tev_bg_hjb_ratio,
     &		CS_tev_ud_hjWp_ratio,CS_tev_cs_hjWp_ratio,
     &		CS_tev_ud_hjWm_ratio,CS_tev_cs_hjWm_ratio,
     &          CS_tev_dd_hjZ_ratio,CS_tev_uu_hjZ_ratio,CS_tev_ss_hjZ_ratio,
     &		CS_tev_cc_hjZ_ratio,CS_tev_bb_hjZ_ratio,
     &          CS_tev_pp_vbf_ratio,
     &		BR_hjbb,BR_hjtautau,BR_hjWW,BR_hjgaga)

! translate CS_lep_hjhi_ratio_compact to CS_lep_hjhi_ratio
        k=0
        do j=1,NHIGGS
          do i=1,NHIGGS
            if(i .le. j) then
              k=k+1
              CS_lep_hjhi_ratio(j,i)=CS_lep_hjhi_ratio_compact(k)
              if(i .ne. j) CS_lep_hjhi_ratio(i,j)=CS_lep_hjhi_ratio(j,i)
            endif
          enddo
        enddo


	call calc_hadCS_from_partCS(NHIGGS, Mh,
     &          CS_tev_gg_hj_ratio, CS_tev_bb_hj_ratio, CS_tev_bg_hjb_ratio,
     &		CS_tev_ud_hjWp_ratio, CS_tev_cs_hjWp_ratio, 
     &		CS_tev_ud_hjWm_ratio, CS_tev_cs_hjWm_ratio,
     &          CS_tev_dd_hjZ_ratio, CS_tev_uu_hjZ_ratio, CS_tev_ss_hjZ_ratio, 
     &		CS_tev_cc_hjZ_ratio, CS_tev_bb_hjZ_ratio,
     &          CS_tev_pp_hj_ratio, CS_tev_pp_hjb_ratio, 
     &		CS_tev_pp_hjW_ratio, CS_tev_pp_hjZ_ratio)

	call run_HiggsBounds_hadr(NHIGGS,Mh,
     & CS_lep_hjZ_ratio,CS_lep_hjhi_ratio,
     & CS_tev_pp_hj_ratio,CS_tev_pp_hjb_ratio,
     & CS_tev_pp_hjW_ratio,CS_tev_pp_hjZ_ratio,
     & CS_tev_pp_vbf_ratio,
     & BR_hjbb,BR_hjtautau,
     & BR_hjWW,BR_hjgaga,
     & BR_hjhihi,
     & HBresult,chan,obsratio, ncombined )

	end

*************************************************************************
	subroutine run_HiggsBounds_part(NHIGGS,Mh, 
     & CS_lep_hjZ_ratio, 
     & CS_lep_hjhi_ratio, 
     & CS_tev_gg_hj_ratio,CS_tev_bb_hj_ratio, 
     & CS_tev_bg_hjb_ratio, 
     & CS_tev_ud_hjWp_ratio,CS_tev_cs_hjWp_ratio, 
     & CS_tev_ud_hjWm_ratio,CS_tev_cs_hjWm_ratio, 
     & CS_tev_dd_hjZ_ratio,CS_tev_uu_hjZ_ratio, 
     & CS_tev_ss_hjZ_ratio,CS_tev_cc_hjZ_ratio, 
     & CS_tev_bb_hjZ_ratio, 
     & CS_tev_pp_vbf_ratio, 
     & BR_hjbb,BR_hjtautau, 
     & BR_hjWW,BR_hjgaga, 
     & BR_hjhihi, 
     & HBresult,chan, 
     & obsratio, ncombined ) 
*************************************************************************
	implicit none
! input:
	integer NHIGGS
	double precision Mh(NHIGGS),GammaTotal(NHIGGS)
        double precision CS_lep_hjZ_ratio(NHIGGS),
     &		CS_lep_hjhi_ratio(NHIGGS,NHIGGS)
	double precision CS_tev_gg_hj_ratio(NHIGGS),CS_tev_bb_hj_ratio(NHIGGS),
     & 		CS_tev_bg_hjb_ratio(NHIGGS),
     & 		CS_tev_ud_hjWp_ratio(NHIGGS),CS_tev_cs_hjWp_ratio(NHIGGS),
     & 		CS_tev_ud_hjWm_ratio(NHIGGS),CS_tev_cs_hjWm_ratio(NHIGGS),
     & 		CS_tev_dd_hjZ_ratio(NHIGGS),CS_tev_uu_hjZ_ratio(NHIGGS),
     & 		CS_tev_ss_hjZ_ratio(NHIGGS),CS_tev_cc_hjZ_ratio(NHIGGS),
     & 		CS_tev_bb_hjZ_ratio(NHIGGS)
       	double precision BR_hjbb(NHIGGS),BR_hjtautau(NHIGGS),BR_hjWW(NHIGGS),
     &		BR_hjgaga(NHIGGS),BR_hjhihi(NHIGGS,NHIGGS)
! output:
	integer HBresult,chan,ncombined
	double precision obsratio
! internal:
!  Tevatron hadronic CS
	double precision CS_tev_pp_hj_ratio(NHIGGS),
     &		CS_tev_pp_hjb_ratio(NHIGGS),CS_tev_pp_hjW_ratio(NHIGGS),
     &		CS_tev_pp_hjZ_ratio(NHIGGS),CS_tev_pp_vbf_ratio(NHIGGS)


	call calc_hadCS_from_partCS(NHIGGS, Mh,
     &          CS_tev_gg_hj_ratio, CS_tev_bb_hj_ratio, CS_tev_bg_hjb_ratio,
     &		CS_tev_ud_hjWp_ratio, CS_tev_cs_hjWp_ratio, 
     &		CS_tev_ud_hjWm_ratio, CS_tev_cs_hjWm_ratio,
     &          CS_tev_dd_hjZ_ratio, CS_tev_uu_hjZ_ratio, CS_tev_ss_hjZ_ratio, 
     &		CS_tev_cc_hjZ_ratio, CS_tev_bb_hjZ_ratio,
     &          CS_tev_pp_hj_ratio, CS_tev_pp_hjb_ratio, 
     &		CS_tev_pp_hjW_ratio, CS_tev_pp_hjZ_ratio)

	call run_HiggsBounds_hadr(NHIGGS,Mh,
     & 		CS_lep_hjZ_ratio,CS_lep_hjhi_ratio,
     & 		CS_tev_pp_hj_ratio,CS_tev_pp_hjb_ratio,
     & 		CS_tev_pp_hjW_ratio,CS_tev_pp_hjZ_ratio,
     & 		CS_tev_pp_vbf_ratio,
     & 		BR_hjbb,BR_hjtautau,
     & 		BR_hjWW,BR_hjgaga,
     & 		BR_hjhihi,
     & 		HBresult,chan,obsratio, ncombined )

	end

*************************************************************************
	subroutine run_HiggsBounds_hadr(NHIGGS,Mh, 
     & CS_lep_hjZ_ratio,CS_lep_hjhi_ratio, 
     & CS_tev_pp_hj_ratio,CS_tev_pp_hjb_ratio, 
     & CS_tev_pp_hjW_ratio,CS_tev_pp_hjZ_ratio, 
     & CS_tev_pp_vbf_ratio, 
     & BR_hjbb,BR_hjtautau, 
     & BR_hjWW,BR_hjgaga, 
     & BR_hjhihi, 
     & HBresult,chan,obsratio,ncombined ) 
*************************************************************************
	implicit none
! input:
	integer NHIGGS
	character*5 whichexpt
        common /chosen_analyses/ whichexpt

	double precision Mh(NHIGGS),GammaTotal(NHIGGS)
        double precision CS_lep_hjZ_ratio(NHIGGS),
     &		CS_lep_hjhi_ratio(NHIGGS,NHIGGS)
	double precision CS_tev_pp_hj_ratio(NHIGGS),
     &		CS_tev_pp_hjb_ratio(NHIGGS),CS_tev_pp_hjW_ratio(NHIGGS),
     &		CS_tev_pp_hjZ_ratio(NHIGGS),CS_tev_pp_vbf_ratio(NHIGGS)
       	double precision BR_hjbb(NHIGGS),BR_hjtautau(NHIGGS),BR_hjWW(NHIGGS),
     &		BR_hjgaga(NHIGGS),BR_hjhihi(NHIGGS,NHIGGS)
! output:
	integer HBresult,chan,ncombined
	double precision obsratio
! internal:
        double precision BR_hjhihi_compact(NHIGGS**2-NHIGGS)
        double precision CS_lep_hjhi_ratio_compact((NHIGGS*(NHIGGS+1))/2)

	logical useLEP,useTEV,useSingH,CMDLINE	
	integer iddummy,sanity_flag,i,j,k

        logical lep_allowed,lep_final_h_comb(NHIGGS)
        integer lep_mlratio
        integer lep_result_flag,lep_num_H_combined,lep_sim_mass_sum(NHIGGS)
        double precision lep_obsratio,lep_predratio,lep_model_R_sig_BR

        logical tev_allowed,tev_final_h_comb(NHIGGS)
        integer tev_mlratio
        integer tev_result_flag,tev_num_H_combined
        double precision tev_obsratio,tev_predratio,tev_model_R_sig_BR

	logical allowed,H_combined(NHIGGS)
        integer mlratio,experiment_id
c experiment_id : 1 : LEP, 2 : Tevatron
        integer sim_mass_sum(NHIGGS)
        double precision predratio,model_R_sig_BR
#include "tables-tev_cb.h"
#include "tables-lep_cb.h"



	call determine_topology_flags(whichexpt,useTEV,useLEP,useSingH)

	iddummy=0
	CMDLINE=.FALSE.
	call check_input_sanity(NHIGGS,CMDLINE,iddummy,useLEP,useTEV,useSingH,
     &		Mh,BR_hjbb,BR_hjtautau,BR_hjWW,BR_hjgaga,BR_hjhihi,
     &		sanity_flag)


        if(sanity_flag .eq. 1) then
!tag1
	  if(useLEP) then
! translate BR_hjhihi to BR_hjhihi_compact
	k=0
        do j=1,NHIGGS
          do i=1,NHIGGS
            if(j .ne. i) then
              k=k+1
              BR_hjhihi_compact(k)=BR_hjhihi(j,i)
            endif
          enddo
        enddo

! translate CS_lep_hjhi_ratio to CS_lep_hjhi_ratio_compact
        k=0
        do j=1,NHIGGS
          do i=1,NHIGGS
            if(i .le. j) then
              k=k+1
              CS_lep_hjhi_ratio_compact(k)=CS_lep_hjhi_ratio(j,i)
            endif
          enddo
        enddo

c check LEP bounds
            call check_LEP_bound(NHIGGS,useSingH,Mh,
     &		CS_lep_hjZ_ratio,CS_lep_hjhi_ratio_compact,
     &          BR_hjbb,BR_hjtautau,BR_hjhihi_compact,
     &          lep_allowed,lep_result_flag,lep_mlratio,lep_obsratio,lep_predratio,
     &		lep_num_H_combined,lep_final_h_comb,lep_model_R_sig_BR)
	  endif

	  if(useTEV) then
c check Tevatron bounds
	    call check_TEV_bound(NHIGGS, Mh, GammaTotal,
     &          CS_tev_pp_hj_ratio,CS_tev_pp_hjb_ratio,CS_tev_pp_hjW_ratio,
     &		CS_tev_pp_hjZ_ratio,CS_tev_pp_vbf_ratio,
     &          BR_hjbb,BR_hjtautau,BR_hjWW,BR_hjgaga,
     &          tev_allowed,tev_result_flag,tev_mlratio,tev_obsratio,
     &		tev_predratio,tev_num_H_combined,tev_final_h_comb,
     &		tev_model_R_sig_BR)
	  endif

	  if(useLEP) then
                experiment_id = 1
                allowed = lep_allowed
                mlratio = lep_mlratio
                obsratio = lep_obsratio
                predratio = lep_predratio
                HBresult = lep_result_flag
	 	ncombined = lep_num_H_combined
          endif
	  if(useTEV) then
                experiment_id = 2
                allowed = tev_allowed
                mlratio = tev_mlratio
                obsratio = tev_obsratio
		predratio = tev_predratio
                HBresult = tev_result_flag
		ncombined = tev_num_H_combined
	  endif

	  if(useLEP .and. useTEV) then
c  decide on which ratio to choose from LEP or TEV and set flag: experiment_id
#ifdef VERBOSE
	    write(*,*)'Overall channel with highest sensitivity:'
#endif
            if(lep_predratio .ge. tev_predratio) then
                experiment_id = 1
                allowed = lep_allowed
                mlratio = lep_mlratio
                obsratio = lep_obsratio
                predratio = lep_predratio
                HBresult = lep_result_flag
		ncombined = lep_num_H_combined
#ifdef VERBOSE
		call write_LEP_channel(NHIGGS,mlratio,Mh)
#endif
            else
                experiment_id = 2
                allowed = tev_allowed
                mlratio = tev_mlratio
                obsratio = tev_obsratio
                predratio = tev_predratio
                HBresult = tev_result_flag
		ncombined = tev_num_H_combined
#ifdef VERBOSE
		call write_TEV_channel(NHIGGS,mlratio,Mh)
#endif
            endif
#ifdef VERBOSE
        	write(*,*)'   R_sig_BR/S95_predicted =',predratio
        	write(*,*)'   R_sig_BR/S95_observed  =',obsratio
#endif
	  endif

	endif
!tag1
	if(sanity_flag .eq. 0) then
		HBresult=-1
		experiment_id=0
		ncombined = 0
	endif

! If obsratio is zero, than output_chan is set to zero.
! Realization: set experiment_id=0
!	       then determine_output_channel_number will set 
!	       output_chan=0
	if(obsratio .eq. 0d0) then
	  experiment_id=0
	  ncombined=0
	endif

	call determine_output_channel_number(NHIGGS,whichexpt,
     &          experiment_id,mlratio,chan)

	end


***************************************************
        subroutine initialize_HiggsBounds(NHIGGS,whichexpt)
***************************************************
        implicit none
        integer NHIGGS
	character*5 whichexpt,whichexpt_val
	common /chosen_analyses/ whichexpt_val

	whichexpt_val=whichexpt

	call write_HiggsBounds_info()

        call initialize_SM_decay_data()
        call initialize_TEV_tables(NHIGGS)
        call initialize_LEP_tables(NHIGGS)

	open(87,file='Key.dat')
	call write_table_key(87,NHIGGS,whichexpt)
	close(87)

        end

***************************************************
        subroutine finish_HiggsBounds()
***************************************************
        implicit none


        end

*************************************************************************
        subroutine write_HiggsBounds_info()
*************************************************************************
        implicit none

	write(*,*)
	write(*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
	write(*,*)'~                                                        ~'
	write(*,*)'~          HiggsBounds 1.0 (fortran77 version)           ~'
	write(*,*)'~                                                        ~'
	write(*,*)'~     Philip Bechtle, Oliver Brein, Sven Heinemeyer,     ~'
	write(*,*)'~            Georg Weiglein, Karina E. Williams          ~'
	write(*,*)'~                                                        ~'
	write(*,*)'~ arXiv:0811.4169, http://www.ippp.dur.ac.uk/HiggsBounds ~'
	write(*,*)'~                                                        ~'
	write(*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
	write(*,*)
	write(*,*)'HiggsBounds collects together results from '
	write(*,*)
	write(*,*)'    * the LEP collaborations and LEP Higgs Working Group'
	write(*,*)'    * the CDF and D0 Collaborations'
	write(*,*)'    * the program HDECAY (arXiv:hep-ph/9704448)'
	write(*,*)'    * TeV4LHC Higgs Working Group report'
	write(*,*)'      (see arXiv:hep-ph/0612172 and ref. therein)'
	write(*,*)

	end




*****************************************************************************
	subroutine determine_topology_flags(whichexpt,
     &		useTEV,useLEP,useSingH)
*****************************************************************************
	implicit none 

	character*5 whichexpt

	integer lnblnk
        external lnblnk

	logical useTEV,useLEP,useSingH


	useSingH = .FALSE.

	if(whichexpt .eq. 'onlyT') then
		useTEV=.TRUE.
		useLEP=.FALSE.
	elseif(whichexpt .eq. 'onlyL') then
		useTEV=.FALSE.
                useLEP=.TRUE.
	elseif(whichexpt .eq. 'LandT') then
                useTEV=.TRUE.
                useLEP=.TRUE.
	elseif(whichexpt .eq. 'singH') then
               	useTEV=.TRUE.
               	useLEP=.TRUE.	
	       	useSingH = .TRUE.
	endif
	
	end


*****************************************************************************
        subroutine check_input_sanity(NHIGGS,CMDLINE,id,useLEP,useTEV,useSingH,
     &		Mh,BRhjbbbar,BRhjtautau,BRhjWW,
     & 		BRhjgamgam,BRhjhihi,sanity_flag)
*****************************************************************************
* check_input_sanity ignores all BRs which are not relevant to the experiment.

	implicit none
	logical useLEP,useTEV,useSingH,CMDLINE
	integer NHIGGS,sanity_flag,i,j,id
	double precision sumBR(NHIGGS)
	double precision Mh(NHIGGS),BRhjbbbar(NHIGGS),BRhjtautau(NHIGGS)
        double precision BRhjWW(NHIGGS),BRhjgamgam(NHIGGS),BRhjhihi(NHIGGS,NHIGGS)
	double precision fuzziness

	fuzziness=0.05d0

	sanity_flag=1
	
	do j=1,NHIGGS
	  sumBR(j)=0d0
	  if(Mh(j) .le. 0d0) then
	    if(CMDLINE) write(*,7788) j,id
	    if(.not. CMDLINE) write(*,7789) j
 7788	    FORMAT('WARNING: zero or negative mass of Higgs #',I3,' in input line',I6)
 7789	    FORMAT('WARNING: zero or negative mass of Higgs #',I3)
            sanity_flag=0
	  endif
	enddo


	do j=1,NHIGGS

!>> check_input_sanity ignores all BRs which are not relevant 
!   to the chosen experiment.
!	  if(useLEP) then 
!		sumBR(j)=BRhjbbbar(j)+BRhjtautau(j)
!	    if(.not. useSingH) then
!	      do i=1,NHIGGS
!                sumBR(j)=sumBR(j)+BRhjhihi(j,i)
!	      enddo
!	    endif
!	  endif
!	  if(useTEV) then
!		sumBR(j)=BRhjbbbar(j)+BRhjtautau(j)
!     &			     +BRhjWW(j)+BRhjgamgam(j)
!	    if(useLEP .and. (.not. useSingH)) then
!	      do i=1,NHIGGS
!                sumBR(j)=sumBR(j)+BRhjhihi(j,i)
!              enddo
!	    endif
!    	  endif
!<<

!>> check_input_sanity takes all input into account
	  sumBR(j)=BRhjbbbar(j)+BRhjtautau(j)+BRhjWW(j)+BRhjgamgam(j)
	  do i=1,NHIGGS
            sumBR(j)=sumBR(j)+BRhjhihi(j,i)
          enddo
!<<

!>> 21.01.2009 fuzziness construction switched off
!          if(sumBR(j) .gt. 1d0+fuzziness) then
!	     if(CMDLINE) write(*,7790) j,id
!	     if(.not. CMDLINE) write(*,7791) j
! 7790	     FORMAT('WARNING: nonsensical sum of BRs for Higgs #',I3,' in input line',I6)
! 7791	     FORMAT('WARNING: nonsensical sum of BRs for Higgs #',I3)
!	     write(*,*)'BRhjbbbar(',j,')=',BRhjbbbar(j)
!            write(*,*)'BRhjtautau(',j,')=',BRhjtautau(j)
!	     write(*,*)'BRhjWW(',j,')=',BRhjWW(j)
!             write(*,*)'BRhjgamgam(',j,')=',BRhjgamgam(j)
!	     write(*,*)'sumBR(',j,')=',sumBR(j)
!	     sanity_flag=0
!	  endif
!<<
          if(sumBR(j) .gt. 1d0) then
	     if(CMDLINE) write(*,7790) sumBR(j),j,id
	     if(.not. CMDLINE) write(*,7791) sumBR(j),j
 7790	     FORMAT('WARNING: sum of BRs larger than 1 (',G10.4,') for Higgs #',I3,' in input line',I6)
 7791	     FORMAT('WARNING: sum of BRs larger than 1 (',G10.4,') for Higgs #',I3)
! 21.01.2009 currently only a warning is given
!	     sanity_flag=0
	  endif
	enddo

	end


*****************************************************************************
        subroutine determine_output_channel_number(NHIGGS,whichexpt,
     &		experiment_id,exp_chan,output_chan)
*****************************************************************************
        implicit none
	character*5 whichexpt
	integer NHIGGS,experiment_id,exp_chan,output_chan,tev_offset

	if(experiment_id .eq. 0) then
		output_chan = 0
	else
	if(whichexpt .eq. 'onlyT') then
		tev_offset=0
		output_chan=exp_chan
	elseif(whichexpt .eq. 'onlyL') then
		output_chan=exp_chan
	elseif(whichexpt .eq. 'LandT') then
		tev_offset=LEP_NSCHN*NHIGGS+LEP_NDCHN*NHIGGS**2
		if(experiment_id .eq. 1) then
			output_chan=exp_chan
		elseif(experiment_id .eq. 2) then
			output_chan=exp_chan+tev_offset
		endif
	elseif(whichexpt .eq. 'singH') then
		tev_offset=LEP_NSCHN*NHIGGS
		if(experiment_id .eq. 1) then
			output_chan=exp_chan
		elseif(experiment_id .eq. 2) then
			output_chan=exp_chan+tev_offset
		endif
	endif
	endif

	end

!******************************************************************
	subroutine write_table_key(file_id,NHIGGS,whichexpt)
!******************************************************************
	implicit none
	character*5 whichexpt
	logical useTEV,useLEP,useSingH
        integer NHIGGS,file_id,m,n,i,j,findi,findj,TEV_reaction_number
        integer LEP_reaction_number,number_LEP_channels,output_chan
	integer channel	

#include "tables-lep_cb.h"
#include "tables-tev_cb.h"

	write(file_id,*)'   0:  no analysis applies'

	call determine_topology_flags(whichexpt,
     &          useTEV,useLEP,useSingH)

	if(useLEP) then

	  if(useSingH) then
		number_LEP_channels = LEP_NSCHN*NHIGGS
	  else
		number_LEP_channels = LEP_NSCHN*NHIGGS+LEP_NDCHN*NHIGGS**2
	  endif
c>> LEP
	  do channel=1,number_LEP_channels

	  n=LEP_reaction_number(NHIGGS,channel)

	  if(channel .le. LEP_NSCHN*NHIGGS) then
	    if(tlist_lep(channel) .eq. 1) then
	      write(file_id,142) channel,n,n
 142          FORMAT ('  ',I3,':  e+ e- -> H(',I1,') Z, H(',I1,') -> b b')
            elseif(tlist_lep(channel) .eq. 2)  then
	      write(file_id,143) channel,n,n
 143          FORMAT ('  ',I3,':  e+ e- -> H(',I1,') Z, H(',I1,') -> tau tau')
	    endif
	  else
	    i=findi(NHIGGS,n)
	    j=findj(NHIGGS,n)
	      if(tlist_lep(channel) .eq. 3) then
		write(file_id,150) channel,j,j,i,i,i
 150 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') Z, H(',I1,') -> H(',I1,') H(',I1,'), H(',I1,') -> b b')
              elseif(tlist_lep(channel) .eq. 4) then
		write(file_id,160) channel,j,j,i,i,i
 160 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') Z, H(',I1,') -> H(',I1,') H(',I1,'), H(',I1,') -> tau tau')
              elseif(tlist_lep(channel) .eq. 5) then
		write(file_id,180) channel,j,i,j,i
 180 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') H(',I1,'),  H(',I1,'), H(',I1,') -> b b')
              elseif(tlist_lep(channel) .eq. 6) then 
		write(file_id,190) channel,j,i,j,i
 190 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') H(',I1,'),  H(',I1,'), H(',I1,') -> tau tau')
              elseif(tlist_lep(channel) .eq. 7) then 
		write(file_id,200) channel,j,i,j,i,i,i
 200 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') H(',I1,'), H(',I1,') -> H(',I1,') H(',I1,'), H(',I1,') -> b b')
              elseif(tlist_lep(channel) .eq. 8) then
		write(file_id,210) channel,j,i,j,i,i,i
 210 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') H(',I1,'), H(',I1,') -> H(',I1,') H(',I1,'), H(',I1,') -> tau tau')
              elseif(tlist_lep(channel) .eq. 9) then
		write(file_id,220) channel,j,j,i,i,i
 220 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') Z, H(',I1,') -> H(',I1,') H(',I1,'), H(',I1,') -> b b, tau tau')
              elseif(tlist_lep(channel) .eq. 10) then
		write(file_id,230) channel,j,i,j,i
 230 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') H(',I1,'), H(',I1,') -> b b, H(',I1,') -> tau tau')
              elseif(tlist_lep(channel) .eq. 11) then
		write(file_id,240) channel,j,i,j,i
 240 		FORMAT ('  ',I3,':  e+ e- -> H(',I1,') H(',I1,'), H(',I1,') -> tau tau, H(',I1,') -> b b')
	      endif
	  endif

	  enddo

	endif
c<< LEP
	if(useTEV) then
	  do channel=1,TEV_NSCHN*NHIGGS+TEV_NDCHN*NHIGGS**2

	  n=TEV_reaction_number(NHIGGS,channel)

	  call determine_output_channel_number(NHIGGS,whichexpt,
     &          2,channel,output_chan)


	  if(channel .le. TEV_NSCHN*NHIGGS) then
	  if(tlist_tev(channel) .eq. CDF0807_4493) write(file_id,1001) output_chan,n
	  if(tlist_tev(channel) .eq. CDF9475) write(file_id,1002) output_chan,n
          if(tlist_tev(channel) .eq. CDF9483) write(file_id,1003) output_chan,n
          if(tlist_tev(channel) .eq. D05570) write(file_id,1004) output_chan,n
          if(tlist_tev(channel) .eq. D05586) write(file_id,1005) output_chan,n
          if(tlist_tev(channel) .eq. CDF8961_D05536) write(file_id,1009) output_chan,n
          if(tlist_tev(channel) .eq. CDF9284) write(file_id,1010) output_chan,n
          if(tlist_tev(channel) .eq. D05472) write(file_id,1011) output_chan,n
          if(tlist_tev(channel) .eq. CDF9463) write(file_id,1013) output_chan,n
          if(tlist_tev(channel) .eq. D05726) write(file_id,1014) output_chan,n
          if(tlist_tev(channel) .eq. D00805_3556) write(file_id,1015) output_chan,n
          if(tlist_tev(channel) .eq. D00805_2491) write(file_id,1017) output_chan,n
          if(tlist_tev(channel) .eq. CDF9071) write(file_id,1018) output_chan,n
          if(tlist_tev(channel) .eq. D05485) write(file_id,1019) output_chan,n
          if(tlist_tev(channel) .eq. CDF7307) write(file_id,1020) output_chan,n
          if(tlist_tev(channel) .eq. D05737) write(file_id,1022) output_chan,n
          if(tlist_tev(channel) .eq. D00803_1514) write(file_id,1023) output_chan,n
          if(tlist_tev(channel) .eq. CDF9248) write(file_id,1024) output_chan,n
          if(tlist_tev(channel) .eq. CDF9290_D05645) write(file_id,1025) output_chan,n
          if(tlist_tev(channel) .eq. D05757) write(file_id,1026) output_chan,n
          if(tlist_tev(channel) .eq. CDF0809_3930) write(file_id,1027) output_chan,n
          if(tlist_tev(channel) .eq. CDF9465_D05754) write(file_id,1028) output_chan,n

 1001	FORMAT ('  ',I3,': p p-bar -> Z H(',I1,') -> l l b b-bar, CDF with 1 fb^-1 (hep-ex/0807.4493)')
 1002	FORMAT ('  ',I3,': p p-bar -> Z H(',I1,') -> l l b b-bar, CDF with 2.4 fb^-1 (CDF note 9475)')
 1003   FORMAT ('  ',I3,': p p-bar -> V H(',I1,') -> b b-bar + miss. E_T (V=W,Z), CDF with 2.1 fb^-1 SM combined (CDF note 9483)')
 1004	FORMAT ('  ',I3,': p p-bar -> Z H(',I1,') -> l l b b-bar, D0 with 2.3 fb^-1 (D0 note 5570)')
 1005	FORMAT ('  ',I3,': p p-bar -> V H(',I1,') -> b b-bar + miss. E_T (V=W,Z), D0 with 2.1 fb^-1 (D0 note 5586)')
 1009	FORMAT ('  ',I3,': p p-bar -> H(',I1,') + X, SM combined (CDF note 8961)')
! via HW -> l nu b b-bar, W W+ W- -> l+- nu l+- nu ..,
!                            HZ -> nu nu b b-bar, l l b b-bar,
!                            H -> W W -> l+- nu l-+ nu, SM combined
!                       (CDF note 8961
 1010	FORMAT ('  ',I3,': p p-bar -> b H(',I1,') -> 3 b jets, CDF with 1.9 fb^-1 (CDF note 9284)')
 1011   FORMAT ('  ',I3,': p p-bar -> W H(',I1,'), H -> b b-bar, D0 (D0 note 5472)')
 1013   FORMAT ('  ',I3,': p p-bar -> W H(',I1,') -> l nu b b-bar, CDF with 2.7 fb^-1 (CDF note 9463)')
 1014   FORMAT ('  ',I3,': p p-bar -> b H(',I1,') -> 3 b jets, D0 with 2.6 fb^-1 (D0 note 5726)')
 1015   FORMAT ('  ',I3,': p p-bar -> b H(',I1,') -> 3 b jets, D0 (hep-ex/0805.3556)')
 1017   FORMAT ('  ',I3,': p p-bar -> H(',I1,') -> tau tau, D0 with 1 fb^-1 absolute limits (D0 hep-ex/0805.2491)')
 1018   FORMAT ('  ',I3,': p p-bar -> H(',I1,') -> tau tau, CDF with 1.8 fb^-1 absolute limits (CDF note 9071)')
 1019   FORMAT ('  ',I3,':  p p-bar -> W H(',I1,') -> 3 Ws, D0 with 1 fb^-1 absolute limits (D0 note 5485)') 
 1020   FORMAT ('  ',I3,':  p p-bar -> W H(',I1,') -> 3 Ws, CDF with 1.9 fb^-1 (CDF note 7307)')
 1022   FORMAT ('  ',I3,': p p-bar -> H(',I1,') -> gamma gamma, D0 with 2.68 fb^-1 (D0 note 5737)')
 1023   FORMAT ('  ',I3,': p p-bar -> H(',I1,') -> gamma gamma, D0 with 1.1 fb^-1 absolute limits (D0 hep-ex/0803.1514)')
 1024	FORMAT ('  ',I3,': p p-bar -> H(',I1,') + X -> tau tau, CDF with 2.0 fb^-1 SM combined (CDF note 9248)')
!via HW with W -> j j, 
!                            HZ with Z -> j j,
!                            single H, VBF, SM combined
!                       (CDF note 9248,
 1025     FORMAT ('  ',I3,': p p-bar -> H(',I1,') + X, SM combined (CDF note 9290, D0 note 5645)')
! via HW -> l nu b b-bar, W W+ W- -> l+- nu l+- nu ..,
!                            HZ -> nu nu b b-bar, l l b b-bar,
!                            H -> tau tau via HW with W -> j j,
!                                             HZ with Z -> j j,
!                                             single H, VBF,
!                            SM combined (CDF note 9290, D0 note 5645)
 1026	FORMAT ('  ',I3,': p p-bar -> H(',I1,') -> W W -> l l, D0 with 3.0 fb^-1 (D0 note 5757)')
 1027   FORMAT ('  ',I3,': p p-bar -> H(',I1,') -> W W, CDF with 3.0 fb^-1 (hep-ex/0809.3930)')
 1028	FORMAT ('  ',I3,': p p-bar -> H(',I1,') + X, SM combined (CDF note 9465, D0 note 5754)')
 

	  else
!>> currently not needed. if needed: edit !
!	  i=findi(NHIGGS,n)
!	  j=findj(NHIGGS,n)
! ......
!<< currently not needed. if needed edit !
	  endif

	  enddo
	endif

	end
